#!/usr/bin/perl

#    Lanternor - Used to study lanterns and day signals
#    Copyright (C) 2011  Jonas Eriksson <zqad@acc.umu.se>
# 
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
# 
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
# 
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

use GD::Image;
use GD::Polygon;
use strict;
use warnings;

my $element_size = 20;

# create a new image
my $im = new GD::Image(500,500);

# allocate some colors
my %c;
$c{white} = $im->colorAllocate(0xFF,0xFF,0xFF);
$c{black} = $im->colorAllocate(0x00,0x00,0x00);
$c{red}   = $im->colorAllocate(0xFF,0x00,0x00);
$c{green} = $im->colorAllocate(0x00,0xFF,0x00);

# make image interlaced
$im->interlaced('true');

our %patterns;
require("./patterns.pm");

sub draw_pattern($$);

draw_pattern($im, $ENV{QUERY_STRING});

# make sure we are writing to a binary stream
binmode STDOUT;

# Convert the image to PNG and print it on standard output
print "Content-type: image/png\n\n";
print $im->png;

sub draw_vertical_patterns(@) {
  my ($im, $py, @vertical_patterns) = @_;

  my ($width, $height) = $im->getBounds();
  my $max_per_range = (($width - 2 * $element_size) / ($#vertical_patterns + 1));

  my $px = int($element_size + rand($max_per_range));
  my %rcache = ();
  while (@vertical_patterns) {
    my $vpat_array = pop(@vertical_patterns);
    foreach my $vpat (@{$vpat_array}) {
      my ($color, $figure, $data) = split(/:/, $vpat);
      if ($figure eq 'circle') {
        $im->filledEllipse($px, $py+$data, $element_size, $element_size, $c{$color});
      }
      elsif ($figure eq 'rect') {
        my ($w, $h) = split(/,/, $data);
        $w *= $element_size;
        $h *= $element_size;
        $im->filledRectangle($px - $w / 2, $py - $h / 2, $px + $w / 2, $py + $h / 2, $c{$color});
      }
      elsif ($figure eq 'poly') {
        my @data = split(/;/, $data);
        my $poly = new GD::Polygon;
        foreach my $d (@data) {
          my ($x, $y) = split(/,/, $d);
          $poly->addPt($px + $x * $element_size, $py + $y * $element_size);
        }
        $im->filledPolygon($poly, $c{$color});
      }
    }
    my $dist = pop(@vertical_patterns);
    next if (!defined($dist));
    my $dist_n;
    if ($dist =~ m/^[0-9]+$/) {
      $dist_n = $dist;
    }
    elsif ($dist =~ m/^r(.?)$/) {
      my $cache_entry = $1;
      if (defined $cache_entry && defined $rcache{$cache_entry}) {
        $dist_n = $rcache{$cache_entry};
      }
      else {
        $dist_n = int(rand($max_per_range - 2 * $element_size) + 4 * $element_size);
        $rcache{$cache_entry} = $dist_n if (defined $cache_entry);
      }
    }
    $px += $dist_n;
  }
}

sub draw_pattern($$) {
  my ($img, $pname) = @_;
  my %p = %{$patterns{$pname}};

  my $fill = $c{black};
  if (defined $p{background}) {
    $fill = $c{$p{background}};
  }
  $im->fill(0, 0, $fill);

  draw_vertical_patterns($im, 50, @{$p{vertical_patterns}});
}
